@@ -23,8 +23,9 @@ module Text.Libyaml
     , decode
     , encodeFile
     , decodeFile
-      -- * Exception
+      -- * Error handling
     , YamlException (..)
+    , YamlMark (..)
     ) where
 
 import qualified Data.ByteString.Internal as B
@@ -44,7 +45,7 @@ import Control.Monad.IO.Class
 import Control.Exception (throwIO, Exception, finally)
 import Control.Applicative
 import Control.Monad.Trans.Resource
-import qualified Data.Conduit as C
+import Data.Conduit hiding (Source, Sink, Conduit)
 import Control.Exception (mask_)
 
 data Event =
@@ -164,8 +165,14 @@ foreign import ccall "get_parser_error_problem"
 foreign import ccall "get_parser_error_context"
     c_get_parser_error_context :: Parser -> IO (Ptr CUChar)
 
-foreign import ccall unsafe "get_parser_error_offset"
-    c_get_parser_error_offset :: Parser -> IO CULong
+foreign import ccall unsafe "get_parser_error_index"
+    c_get_parser_error_index :: Parser -> IO CULong
+
+foreign import ccall unsafe "get_parser_error_line"
+    c_get_parser_error_line :: Parser -> IO CULong
+
+foreign import ccall unsafe "get_parser_error_column"
+    c_get_parser_error_column :: Parser -> IO CULong
 
 makeString :: MonadIO m => (a -> m (Ptr CUChar)) -> a -> m String
 makeString f a = do
@@ -452,9 +459,10 @@ newtype ToEventRawException = ToEventRawException CInt
     deriving (Show, Typeable)
 instance Exception ToEventRawException
 
-decode :: C.MonadResource m => B.ByteString -> C.Source m Event
+decode :: MonadResource m => B.ByteString -> GSource m Event
+decode bs | B8.null bs = return ()
 decode bs =
-    C.sourceIO alloc cleanup (runParser . fst)
+    bracketP alloc cleanup (runParser . fst)
   where
     alloc = mask_ $ do
         ptr <- mallocBytes parserSize
@@ -475,9 +483,9 @@ decode bs =
         c_yaml_parser_delete ptr
         free ptr
 
-decodeFile :: C.MonadResource m => FilePath -> C.Source m Event
+decodeFile :: MonadResource m => FilePath -> GSource m Event
 decodeFile file =
-    C.sourceIO alloc cleanup (runParser . fst)
+    bracketP alloc cleanup (runParser . fst)
   where
     alloc = mask_ $ do
         ptr <- mallocBytes parserSize
@@ -506,16 +514,16 @@ decodeFile file =
         c_yaml_parser_delete ptr
         free ptr
 
-runParser :: C.MonadResource m => Parser -> m (C.SourceIOResult Event)
-runParser parser = liftIO $ do
-    e <- parserParseOne' parser
+runParser :: MonadResource m => Parser -> GSource m Event
+runParser parser = do
+    e <- liftIO $ parserParseOne' parser
     case e of
-        Left err -> throwIO $ YamlException err
-        Right Nothing -> return $ C.IOClosed
-        Right (Just ev) -> return $ C.IOOpen ev
+        Left err -> liftIO $ throwIO err
+        Right Nothing -> return ()
+        Right (Just ev) -> yield ev >> runParser parser
 
 parserParseOne' :: Parser
-                -> IO (Either String (Maybe Event))
+                -> IO (Either YamlException (Maybe Event))
 parserParseOne' parser = allocaBytes eventSize $ \er -> do
     res <- liftIO $ c_yaml_parser_parse parser er
     flip finally (c_yaml_event_delete er) $
@@ -523,19 +531,14 @@ parserParseOne' parser = allocaBytes eventSize $ \er -> do
         then do
           problem <- makeString c_get_parser_error_problem parser
           context <- makeString c_get_parser_error_context parser
-          offset <- c_get_parser_error_offset parser
-          return $ Left $ concat
-            [ "YAML parse error: "
-            , problem
-            , "\nContext: "
-            , context
-            , "\nOffset: "
-            , show offset
-            , "\n"
-            ]
+          index <- c_get_parser_error_index parser
+          line <- c_get_parser_error_line parser
+          column <- c_get_parser_error_column parser
+          let problemMark = YamlMark (fromIntegral index) (fromIntegral line) (fromIntegral column)
+          return $ Left $ YamlParseException problem context problemMark
         else Right <$> getEvent er
 
-encode :: C.MonadResource m => C.Sink Event m ByteString
+encode :: MonadResource m => GSink Event m ByteString
 encode =
     runEmitter alloc close
   where
@@ -544,37 +547,34 @@ encode =
         withForeignPtr fbuf c_buffer_init
         withForeignPtr fbuf $ c_my_emitter_set_output emitter
         return fbuf
-    close fbuf = withForeignPtr fbuf $ \b -> do
+    close _ fbuf = withForeignPtr fbuf $ \b -> do
         ptr' <- c_get_buffer_buff b
         len <- c_get_buffer_used b
         fptr <- newForeignPtr_ $ castPtr ptr'
         return $ B.fromForeignPtr fptr 0 $ fromIntegral len
 
-encodeFile :: C.MonadResource m
+encodeFile :: MonadResource m
            => FilePath
-           -> C.Sink Event m ()
+           -> GInfSink Event m
 encodeFile filePath =
-    C.PipeM msink (return ())
+    bracketP getFile c_fclose $ \file -> runEmitter (alloc file) (\u _ -> return u)
   where
-    msink = do
-        (_releaseKey, file) <- flip allocate c_fclose $ do
-            file <- liftIO $ withCString filePath $
-                        \filePath' -> withCString "w" $
-                        \w' -> c_fopen filePath' w'
-            if (file == nullPtr)
-                then throwIO $ YamlException $ "could not open file for write: " ++ filePath
-                else return file
-        return $ runEmitter (alloc file) (return) -- FIXME close file early
-    alloc file emitter = do
-        c_yaml_emitter_set_output_file emitter file
-        return ()
+    getFile = do
+        file <- withCString filePath $
+                    \filePath' -> withCString "w" $
+                    \w' -> c_fopen filePath' w'
+        if (file == nullPtr)
+            then throwIO $ YamlException $ "could not open file for write: " ++ filePath
+            else return file
 
-runEmitter :: C.MonadResource m
+    alloc file emitter = c_yaml_emitter_set_output_file emitter file
+
+runEmitter :: MonadResource m
            => (Emitter -> IO a) -- ^ alloc
-           -> (a -> IO b) -- ^ close
-           -> C.Sink Event m b
+           -> (u -> a -> IO b) -- ^ close
+           -> Pipe l Event o u m b
 runEmitter allocI closeI =
-    C.sinkIO alloc cleanup push close
+    bracketP alloc cleanup go
   where
     alloc = mask_ $ do
         emitter <- mallocBytes emitterSize
@@ -585,11 +585,22 @@ runEmitter allocI closeI =
     cleanup (emitter, _) = do
         c_yaml_emitter_delete emitter
         free emitter
-    push (emitter, _) e = do
-        _ <- liftIO $ toEventRaw e $ c_yaml_emitter_emit emitter
-        return C.IOProcessing
-    close (_, a) = liftIO $ closeI a
+
+    go (emitter, a) =
+        loop
+      where
+        loop = awaitE >>= either close push
+        push e = do
+            _ <- liftIO $ toEventRaw e $ c_yaml_emitter_emit emitter
+            loop
+        close u = liftIO $ closeI u a
+
+-- | The pointer position
+data YamlMark = YamlMark { yamlIndex :: Int, yamlLine :: Int, yamlColumn :: Int }
+    deriving Show
 
 data YamlException = YamlException String
+                   -- | problem, context, index, position line, position column
+                   | YamlParseException { yamlProblem :: String, yamlContext :: String, yamlProblemMark :: YamlMark }
     deriving (Show, Typeable)
 instance Exception YamlException
