@@ -14,8 +14,7 @@ import qualified Data.Conduit.List as CL
 import System.Directory
 import Control.Monad
 import Control.Exception (try, SomeException)
-import Test.Hspec.Monadic
-import Test.Hspec.HUnit ()
+import Test.Hspec
 
 import qualified Data.Yaml as D
 import Data.Yaml (object, array)
@@ -24,7 +23,7 @@ import qualified Data.HashMap.Strict as M
 import qualified Data.Text as T
 
 main :: IO ()
-main = hspecX $ do
+main = hspec $ do
     describe "streaming" $ do
         it "count scalars with anchor" caseCountScalarsWithAnchor
         it "count sequences with anchor" caseCountSequencesWithAnchor
@@ -56,6 +55,13 @@ main = hspecX $ do
     describe "numbers" $ do
         it "parses as string when quoted" caseQuotedNumber
         it "parses as number when unquoted" caseUnquotedNumber
+        it "parses as number !!str is present" caseAttribNumber
+    describe "booleans" $ do
+        it "parses when all lowercase" caseLowercaseBool
+        it "parses when all uppercase" caseUppercaseBool
+        it "parses when titlecase" caseTitlecaseBool
+    describe "empty input" $ do
+        it "doesn't crash" caseEmptyInput
 
 counter :: Monad m => (Y.Event -> Bool) -> C.Sink Y.Event m Int
 counter pred' =
@@ -288,9 +294,9 @@ caseMergeSequence = do
     isJust maybeRes @? "decoder should return Just YamlObject but returned Nothing"
     let res = fromJust maybeRes
     mappingKey res "foo1" @?= (mkScalar "foo")
-    mappingKey res "k1" @?= (D.Number 1)
-    mappingKey res "k2" @?= (D.Number 2)
-    mappingKey res "k3" @?= (D.Number 4)
+    mappingKey res "k1" @?= (D.String "1")
+    mappingKey res "k2" @?= (D.String "2")
+    mappingKey res "k3" @?= (D.String "4")
 
 caseDataTypes :: Assertion
 caseDataTypes =
@@ -305,6 +311,18 @@ caseDataTypes =
         , ("null", D.Null)
         ]
 
-caseQuotedNumber, caseUnquotedNumber :: Assertion
+caseQuotedNumber, caseUnquotedNumber, caseAttribNumber :: Assertion
 caseQuotedNumber = D.decode "foo: \"1234\"" @?= Just (object [("foo", D.String "1234")])
 caseUnquotedNumber = D.decode "foo: 1234" @?= Just (object [("foo", D.Number 1234)])
+caseAttribNumber = D.decode "foo: !!str 1234" @?= Just (object [("foo", D.String "1234")])
+
+obj :: Maybe D.Value
+obj = Just (object [("foo", D.Bool False), ("bar", D.Bool True), ("baz", D.Bool True)])
+
+caseLowercaseBool, caseUppercaseBool, caseTitlecaseBool :: Assertion
+caseLowercaseBool = D.decode "foo: off\nbar: y\nbaz: true" @?= obj
+caseUppercaseBool = D.decode "foo: FALSE\nbar: Y\nbaz: ON" @?= obj
+caseTitlecaseBool = D.decode "foo: No\nbar: Yes\nbaz: True" @?= obj
+
+caseEmptyInput :: Assertion
+caseEmptyInput = D.decode B8.empty @?= (Nothing :: Maybe D.Value)
